#! /usr/bin/env python
"""
argparse_extensions.py

Created by Lucas Swanson
Copyright (c) 2013 Canada's Michael Smith Genome Sciences Centre. All rights reserved.

Extensions to the builtin argparse module, including custom ArgumentParser
objects and argparse.Action subclasses.
"""

# import standard modules
from argparse import ArgumentParser

# import custom modules
from version import VERSION


def common_parser():
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        '--version', action='version', version='%(prog)s {0}'.format(VERSION))
    misc_group = parser.add_argument_group('miscellaneous arguments')
    misc_group.add_argument(
        '-f', '--force', action='store_true',
        help='Force script to run, even if the output directory/files '
             'already exist.')
    misc_group.add_argument(
        '-d', '--debug', action='store_true',
        help='Print debug information while the program runs.')
    misc_group.add_argument(
        '--extreme-debug', action='store_true', dest='extreme_debug',
        help='Print extremely in-depth debug information while the program '
             'runs. Not recommended for large jobs.')
    return parser
