#! /usr/bin/env python

# import standard modules
from optparse import OptionParser, OptionGroup
import os, re, sys, time, traceback

# import custom modules
from version import VERSION
from utils.log import GetLogPath, CloseLogFile
from utils.error import MyError
from utils.general import (SetupMainClass, TimeSpent, WriteCommand)
from utils.messages import ErrMsg, LogMsg, DebugMsg, ExtremeDebugMsg
from utils.files_paths import (CheckFilePath, CheckDirPath, EnsureAbsPath,
  GetOutDir, CheckNewFilePath)
from parsers.candidate_group_parser import CandidateGroupParserCls

# CONSTANTS
ES_SUCCESS = 0
ES_RUN_ERR = 1
ES_OPT_ERR = 2
ES_PATH_ERR = 3
ES_EXCEPTION = 4
MSG_SUCCESS = "SUCCESS"
MSG_FAIL = "FAIL"

class MainClassCls: #{
  def __init__(self, options): #{
    SetupMainClass(self, options)
  #} end def

  def __del__(self): #{
    CloseLogFile(self)
  #} end def

  def Run(self): #{
    LogMsg(self, "Creating pretty file...")
    start = time.time()
    parser = CandidateGroupParserCls(self.options.input_path)
    if (self.options.to_file): #{
      outfile = FileBoxCls(self.options.output_path, "w",
          "cannot create output file")
    #} end if
    for group in parser: #{
      if (self.options.to_file): #{
        outfile.Write(group.FullDataString(readable=True))
      else:
        print group.FullDataString(readable=True),
      #} end if
    #} end for
    LogMsg(self, "Time spent creating pretty file: %s" %
      TimeSpent(start))
  #} end def
#} end class

#### EXCEPTION CLASSES ####
class MainClassError(MyError): #{
  pass
#} end class

def SetupOptionsParser(): #{
  description_string = ("Convert a Barnacle data file to a more readable "
    "format.")
  args = [ "INPUT_FILE", ]
  usage_string = "%prog " + " ".join(args) + " [ OPTIONS ]"
  parser = OptionParser(description=description_string,
                        version="%prog " + VERSION,
                        usage=usage_string)
  parser.num_args = len(args)
  parser.add_option("-o", "--output-to-file",
      action="store_true", dest="to_file",
      help="Write pretty ouput to INPUT_FILE.pretty, rather than stdout.")
  parser.add_option("-O", "--output-path",
      metavar="OUTPUT_FILE",
      help="Write pretty ouput to OUTPUT_FILE, rather than stdout.")
  misc_group = OptionGroup(parser, "Miscellaneous Options")
  misc_group.add_option("-f", "--force",
                    action="store_true",
                    help="Force filtering to take place, even if the output "
                         "directory already exists.")
  misc_group.add_option("-d", "--debug",
                    action="store_true",
                    help="Print debug information while the program runs.")
  misc_group.add_option("--extreme-debug",
                    action="store_true", dest="extreme_debug",
                    help="Print extremely in-depth debug information while "
                      "the program runs. Not recommended for large jobs.")
  parser.add_option_group(misc_group)
  parser.set_defaults(to_file=False,
                      force=False,
                      debug=False,
                      extreme_debug=False)
  return parser
#} end def

def CheckPaths(options): #{
  opts_good = True
  if (not opts_good): #{
    ErrMsg("bad option") #TODO
    opts_good = False
  #} end if
  path_errors = list()
  CheckFilePath(options.input_path, "Barnacle data file", path_errors) #TODO
  # get and check the output path
  if (options.to_file or None != options.output_path): #{
    options.to_file = True
    if (None == options.output_path): #{
      options.output_path = "%s.pretty" % options.input_path
    #} end if
    if (not options.force): #{
      CheckNewFilePath(options.output_path, "Pretty Barnacle file",
          path_errors)
    #} end if
  #} end if
  #options.output_dir = GetOutDir(os.path.dirname(options.input_path),
  #  "TASK_DESC") #TODO
  #if (opts_good and 0 == len(path_errors)): #{
  #  CheckDirPath(options.output_dir, "output", path_errors,
  #    create=True, replace=options.force)
  #  # get the log file name
  #  options.log_file_name = GetLogPath(options.input_path,
  #    "TASK_DESC", options.output_dir) #TODO
  #} end if
  if (0 < len(path_errors)): #{
    ErrMsg("Errors in input arguments:\n  %s" % "\n  ".join(path_errors))
  #} end if
  # the paths are good if there are no path errors and no conflicting options
  return (opts_good and 0 == len(path_errors))
#} end def

def Main(): #{
  # get options and arguments
  parser = SetupOptionsParser()
  (options, args) = parser.parse_args()
  # if the right number of args was used
  if (parser.num_args == len(args)): #{
    options.input_path = EnsureAbsPath(args[0])
    if (CheckPaths(options)): #{
      try: #{
        main_class_object = MainClassCls(options)
        WriteCommand(main_class_object, sys.argv)
        main_class_object.Run()
      except (MyError), e:
        ErrMsg("ERROR while creating pretty file:\n  %s" % e) #TODO
        return ES_RUN_ERR
      #} end try
    else:
      return ES_PATH_ERR
    #} end if
  else:
    parser.error("you must specify the path to a Barnacle data file "
        "(INPUT_FILE).")
    return ES_OPT_ERR
  #} end if
  return ES_SUCCESS
#} end def

if __name__ == '__main__': #{
  try: #{
    exit_status = Main()
  except Exception, e:
    traceback.print_exc()
    exit_status = ES_EXCEPTION
  #} end try
  if (ES_SUCCESS == exit_status): #{
    print MSG_SUCCESS
  else:
    print MSG_FAIL
  #} end if
  sys.exit(exit_status)
#} end if
