#! /usr/bin/env python
"""
tool_wrappers.py

Created by Lucas Swanson
Copyright (c) 2013 Canada's Michael Smith Genome Sciences Centre. All rights reserved.
"""

# import standard modules
import os

# import custom modules
from utils.error import MyError
from utils.general import GetCommand
from utils.messages import LogMsg, DebugMsg, ExtremeDebugMsg
from utils.subprocesses import RunCommandFromString

# CONSTANTS

# a safe way to get the value of the "disable profiling timer" (dpt) option
def DPT(self): #{
  if (hasattr(self, "options")): #{
    return getattr(self.options, "dpt", False)
  #} end if
  return False
#} end def

def GetGmapTargetNameFromPath(target_path): #{
  return os.path.splitext(os.path.split(target_path)[-1])[0]
#} end def

def GmapWrapper(self, query_path, target_name, output_path,
    chimera_search=False): #{
  # gmap QUERY -d GENOME -A -f samse
  gmap_cmd = " ".join((GetCommand(self, "gmap"), query_path,
    "-d", target_name, "-A -f samse"))
  ExtremeDebugMsg(self, "%s > %s" % (gmap_cmd, output_path))
  # TODO need to specify stdout file!
  RunCommandFromString(gmap_cmd, stdout=output_path, dpt=DPT(self))
#} end def
